@@ -36,25 +36,28 @@ else:  # pragma: no cover
     input_ = input
 
 
-__version__ = '0.2.0'
+__version__ = '0.3.0'
 __author__ = 'pahaz'
 
 
-DEFAULT_LOGLEVEL = logging.ERROR  #: default level if no logger passed (ERROR)
-TUNNEL_TIMEOUT = 1.0  #: Timeout (seconds) for tunnel connection
+#: Timeout (seconds) for transport socket (``socket.settimeout``)
+SSH_TIMEOUT = 0.1  # ``None`` may cause a block of transport thread
+#: Timeout (seconds) for tunnel connection (open_channel timeout)
+TUNNEL_TIMEOUT = 1.0
+
 _DAEMON = False  #: Use daemon threads in connections
-TRACE_LEVEL = 1
 _CONNECTION_COUNTER = 1
 _LOCK = threading.Lock()
-#: Timeout (seconds) for the connection to the SSH gateway, ``None`` to disable
-SSH_TIMEOUT = None
-DEPRECATIONS = {
+_DEPRECATIONS = {
     'ssh_address': 'ssh_address_or_host',
     'ssh_host': 'ssh_address_or_host',
     'ssh_private_key': 'ssh_pkey',
     'raise_exception_if_any_forwarder_have_a_problem': 'mute_exceptions'
 }
 
+# logging
+DEFAULT_LOGLEVEL = logging.ERROR  #: default level if no logger passed (ERROR)
+TRACE_LEVEL = 1
 logging.addLevelName(TRACE_LEVEL, 'TRACE')
 
 if os.name == 'posix':
@@ -190,7 +193,7 @@ def create_logger(logger=None,
         :class:`logging.Logger`
     """
     logger = logger or logging.getLogger(
-        '{0}.SSHTunnelForwarder'.format(__name__)
+        'sshtunnel.SSHTunnelForwarder'
     )
     if not any(isinstance(x, logging.Handler) for x in logger.handlers):
         logger.setLevel(loglevel or DEFAULT_LOGLEVEL)
@@ -307,16 +310,26 @@ class _ForwardHandler(socketserver.BaseRequestHandler):
             if self.request in rqst:
                 data = self.request.recv(1024)
                 if not data:
+                    self.logger.log(
+                        TRACE_LEVEL,
+                        '>>> OUT {0} recv empty data >>>'.format(self.info)
+                    )
                     break
-                self.logger.log(TRACE_LEVEL,
-                                '>>> OUT {0} send to {1}: {2} >>>'.format(
-                                    self.info,
-                                    self.remote_address,
-                                    hexlify(data)
-                                ))
+                self.logger.log(
+                    TRACE_LEVEL,
+                    '>>> OUT {0} send to {1}: {2} >>>'.format(
+                        self.info,
+                        self.remote_address,
+                        hexlify(data)
+                    )
+                )
                 chan.sendall(data)
             if chan in rqst:  # else
                 if not chan.recv_ready():
+                    self.logger.log(
+                        TRACE_LEVEL,
+                        '<<< IN {0} recv is not ready <<<'.format(self.info)
+                    )
                     break
                 data = chan.recv(1024)
                 self.logger.log(
@@ -339,15 +352,12 @@ class _ForwardHandler(socketserver.BaseRequestHandler):
                 src_addr=src_address,
                 timeout=TUNNEL_TIMEOUT
             )
-        except paramiko.SSHException:
-            chan = None
-        if chan is None:
-            msg = '{0} to {1} was rejected by the SSH server'.format(
-                self.info,
-                self.remote_address
-            )
-            self.logger.log(TRACE_LEVEL, msg)
-            raise HandlerSSHTunnelForwarderError(msg)
+        except Exception as e:
+            msg_tupe = 'ssh ' if isinstance(e, paramiko.SSHException) else ''
+            exc_msg = 'open new channel {0}error: {1}'.format(msg_tupe, e)
+            log_msg = '{0} {1}'.format(self.info, exc_msg)
+            self.logger.log(TRACE_LEVEL, log_msg)
+            raise HandlerSSHTunnelForwarderError(exc_msg)
 
         self.logger.log(TRACE_LEVEL, '{0} connected'.format(self.info))
         try:
@@ -376,14 +386,23 @@ class _ForwardServer(socketserver.TCPServer):  # Not Threading
 
     def __init__(self, *args, **kwargs):
         self.logger = create_logger(kwargs.pop('logger', None))
-        self.tunnel_ok = queue.Queue()
+        self.tunnel_ok = queue.Queue(1)
         socketserver.TCPServer.__init__(self, *args, **kwargs)
 
     def handle_error(self, request, client_address):
         (exc_class, exc, tb) = sys.exc_info()
-        self.logger.error('Could not establish connection from {0} to remote '
-                          'side of the tunnel'.format(request.getsockname()))
-        self.tunnel_ok.put(False)
+        local_side = request.getsockname()
+        remote_side = self.remote_address
+        self.logger.error('Could not establish connection from local {0} '
+                          'to remote {1} side of the tunnel: {2}'
+                          .format(local_side, remote_side, exc))
+        try:
+            self.tunnel_ok.put(False, block=False, timeout=0.1)
+        except queue.Full:
+            # wait untill tunnel_ok.get is called
+            pass
+        except exc:
+            self.logger.error('unexpected internal error: {0}'.format(exc))
 
     @property
     def local_address(self):
@@ -425,7 +444,7 @@ class _UnixStreamForwardServer(UnixStreamServer):
 
     def __init__(self, *args, **kwargs):
         self.logger = create_logger(kwargs.pop('logger', None))
-        self.tunnel_ok = queue.Queue()
+        self.tunnel_ok = queue.Queue(1)
         UnixStreamServer.__init__(self, *args, **kwargs)
 
     @property
@@ -648,9 +667,10 @@ class SSHTunnelForwarder(object):
             Interval in seconds defining the period in which, if no data
             was sent over the connection, a *'keepalive'* packet will be
             sent (and ignored by the remote host). This can be useful to
-            keep connections alive over a NAT
+            keep connections alive over a NAT. You can set to 0.0 for
+            disable keepalive.
 
-            Default: 0.0 (no keepalive packets are sent)
+            Default: 5.0 (no keepalive packets are sent)
 
             .. versionadded:: 0.0.7
 
@@ -746,12 +766,59 @@ class SSHTunnelForwarder(object):
                                 'target can be a valid UNIX domain socket.')
             return False
 
-        if self.skip_tunnel_checkup:  # force tunnel check at this point
-            self.skip_tunnel_checkup = False
-            self.check_tunnels()
-            self.skip_tunnel_checkup = True  # roll it back
+        self.check_tunnels()
         return self.tunnel_is_up.get(target, True)
 
+    def check_tunnels(self):
+        """
+        Check that if all tunnels are established and populates
+        :attr:`.tunnel_is_up`
+        """
+        skip_tunnel_checkup = self.skip_tunnel_checkup
+        try:
+            # force tunnel check at this point
+            self.skip_tunnel_checkup = False
+            for _srv in self._server_list:
+                self._check_tunnel(_srv)
+        finally:
+            self.skip_tunnel_checkup = skip_tunnel_checkup  # roll it back
+
+    def _check_tunnel(self, _srv):
+        """ Check if tunnel is already established """
+        if self.skip_tunnel_checkup:
+            self.tunnel_is_up[_srv.local_address] = True
+            return
+        self.logger.info('Checking tunnel to: {0}'.format(_srv.remote_address))
+        if isinstance(_srv.local_address, string_types):  # UNIX stream
+            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+        else:
+            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        s.settimeout(TUNNEL_TIMEOUT)
+        try:
+            # Windows raises WinError 10049 if trying to connect to 0.0.0.0
+            connect_to = ('127.0.0.1', _srv.local_port) \
+                if _srv.local_host == '0.0.0.0' else _srv.local_address
+            s.connect(connect_to)
+            self.tunnel_is_up[_srv.local_address] = _srv.tunnel_ok.get(
+                timeout=TUNNEL_TIMEOUT * 1.1
+            )
+            self.logger.debug(
+                'Tunnel to {0} is DOWN'.format(_srv.remote_address)
+            )
+        except socket.error:
+            self.logger.debug(
+                'Tunnel to {0} is DOWN'.format(_srv.remote_address)
+            )
+            self.tunnel_is_up[_srv.local_address] = False
+
+        except queue.Empty:
+            self.logger.debug(
+                'Tunnel to {0} is UP'.format(_srv.remote_address)
+            )
+            self.tunnel_is_up[_srv.local_address] = True
+        finally:
+            s.close()
+
     def _make_ssh_forward_handler_class(self, remote_address_):
         """
         Make SSH Handler class
@@ -825,7 +892,7 @@ class SSHTunnelForwarder(object):
             mute_exceptions=False,
             remote_bind_address=None,
             remote_bind_addresses=None,
-            set_keepalive=0.0,
+            set_keepalive=5.0,
             threaded=True,  # old version False
             compression=None,
             allow_agent=True,  # look for keys from an SSH agent
@@ -1116,12 +1183,17 @@ class SSHTunnelForwarder(object):
             _socket.settimeout(SSH_TIMEOUT)
             _socket.connect((self.ssh_host, self.ssh_port))
         transport = paramiko.Transport(_socket)
-        if isinstance(transport.sock, socket.socket):
-            transport.sock.settimeout(SSH_TIMEOUT)
+        sock = transport.sock
+        if isinstance(sock, socket.socket):
+            sock.settimeout(SSH_TIMEOUT)
         transport.set_keepalive(self.set_keepalive)
         transport.use_compression(compress=self.compression)
         transport.daemon = self.daemon_transport
-
+        if isinstance(sock, socket.socket):
+            sock_timeout = sock.gettimeout()
+            sock_info = repr((sock.family, sock.type, sock.proto))
+            self.logger.debug('Transport socket info: {0}, timeout={1}'
+                              .format(sock_info, sock_timeout))
         return transport
 
     def _create_tunnels(self):
@@ -1178,19 +1250,19 @@ class SSHTunnelForwarder(object):
         """
         Processes optional deprecate arguments
         """
-        if deprecated_attrib not in DEPRECATIONS:
+        if deprecated_attrib not in _DEPRECATIONS:
             raise ValueError('{0} not included in deprecations list'
                              .format(deprecated_attrib))
         if deprecated_attrib in kwargs:
             warnings.warn("'{0}' is DEPRECATED use '{1}' instead"
                           .format(deprecated_attrib,
-                                  DEPRECATIONS[deprecated_attrib]),
+                                  _DEPRECATIONS[deprecated_attrib]),
                           DeprecationWarning)
             if attrib:
                 raise ValueError("You can't use both '{0}' and '{1}'. "
                                  "Please only use one of them"
                                  .format(deprecated_attrib,
-                                         DEPRECATIONS[deprecated_attrib]))
+                                         _DEPRECATIONS[deprecated_attrib]))
             else:
                 return kwargs.pop(deprecated_attrib)
         return attrib
@@ -1241,50 +1313,6 @@ class SSHTunnelForwarder(object):
                                  .format(pkey_file, pkey_class))
         return ssh_pkey
 
-    def _check_tunnel(self, _srv):
-        """ Check if tunnel is already established """
-        if self.skip_tunnel_checkup:
-            self.tunnel_is_up[_srv.local_address] = True
-            return
-        self.logger.info('Checking tunnel to: {0}'.format(_srv.remote_address))
-        if isinstance(_srv.local_address, string_types):  # UNIX stream
-            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
-        else:
-            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        s.settimeout(TUNNEL_TIMEOUT)
-        try:
-            # Windows raises WinError 10049 if trying to connect to 0.0.0.0
-            connect_to = ('127.0.0.1', _srv.local_port) \
-                if _srv.local_host == '0.0.0.0' else _srv.local_address
-            s.connect(connect_to)
-            self.tunnel_is_up[_srv.local_address] = _srv.tunnel_ok.get(
-                timeout=TUNNEL_TIMEOUT * 1.1
-            )
-            self.logger.debug(
-                'Tunnel to {0} is DOWN'.format(_srv.remote_address)
-            )
-        except socket.error:
-            self.logger.debug(
-                'Tunnel to {0} is DOWN'.format(_srv.remote_address)
-            )
-            self.tunnel_is_up[_srv.local_address] = False
-
-        except queue.Empty:
-            self.logger.debug(
-                'Tunnel to {0} is UP'.format(_srv.remote_address)
-            )
-            self.tunnel_is_up[_srv.local_address] = True
-        finally:
-            s.close()
-
-    def check_tunnels(self):
-        """
-        Check that if all tunnels are established and populates
-        :attr:`.tunnel_is_up`
-        """
-        for _srv in self._server_list:
-            self._check_tunnel(_srv)
-
     def start(self):
         """ Start the SSH tunnels """
         if self.is_alive:
@@ -1308,9 +1336,18 @@ class SSHTunnelForwarder(object):
             self._raise(HandlerSSHTunnelForwarderError,
                         'An error occurred while opening tunnels.')
 
-    def stop(self):
+    def stop(self, force=False):
         """
-        Shut the tunnel down.
+        Shut the tunnel down. By default we are always waiting until closing
+        all connections. You can use `force=True` to force close connections
+
+        Keyword Arguments:
+            force (bool):
+                Force close current connections
+
+                Default: False
+
+                .. versionadded:: 0.2.2
 
         .. note:: This **had** to be handled with care before ``0.1.0``:
 
@@ -1331,7 +1368,7 @@ class SSHTunnelForwarder(object):
             (address_to_str(k.local_address) for k in self._server_list)
         ) or 'None'
         self.logger.debug('Listening tunnels: ' + opened_address_text)
-        self._stop_transport()
+        self._stop_transport(force=force)
         self._server_list = []  # reset server list
         self.tunnel_is_up = {}  # reset tunnel status
 
@@ -1396,18 +1433,26 @@ class SSHTunnelForwarder(object):
             address_to_str(_srv.remote_address))
         )
 
-    def _stop_transport(self):
+    def _stop_transport(self, force=False):
         """ Close the underlying transport when nothing more is needed """
         try:
             self._check_is_started()
         except (BaseSSHTunnelForwarderError,
                 HandlerSSHTunnelForwarderError) as e:
             self.logger.warning(e)
+        if force and self.is_active:
+            # don't wait connections
+            self.logger.info('Closing ssh transport')
+            self._transport.close()
+            self._transport.stop_thread()
         for _srv in self._server_list:
-            tunnel = _srv.local_address
-            if self.tunnel_is_up[tunnel]:
-                self.logger.info('Shutting down tunnel {0}'.format(tunnel))
-                _srv.shutdown()
+            status = 'up' if self.tunnel_is_up[_srv.local_address] else 'down'
+            self.logger.info('Shutting down tunnel: {0} <> {1} ({2})'.format(
+                address_to_str(_srv.local_address),
+                address_to_str(_srv.remote_address),
+                status
+            ))
+            _srv.shutdown()
             _srv.server_close()
             # clean up the UNIX domain socket if we're using one
             if isinstance(_srv, _UnixStreamForwardServer):
@@ -1418,6 +1463,7 @@ class SSHTunnelForwarder(object):
                                       .format(self.local_address, repr(e)))
         self.is_alive = False
         if self.is_active:
+            self.logger.info('Closing ssh transport')
             self._transport.close()
             self._transport.stop_thread()
         self.logger.debug('Transport is closed')
@@ -1557,7 +1603,7 @@ class SSHTunnelForwarder(object):
             self.__exit__()
 
     def __exit__(self, *args):
-        self._stop_transport()
+        self.stop(force=True)
 
 
 def open_tunnel(*args, **kwargs):
@@ -1581,12 +1627,6 @@ def open_tunnel(*args, **kwargs):
 
             .. versionadded:: 0.1.0
 
-        block_on_close (boolean):
-            Wait until all connections are done during close by changing the
-            value of :attr:`~SSHTunnelForwarder.block_on_close`
-
-            Default: True
-
     .. note::
         A value of ``debug_level`` set to 1 == ``TRACE`` enables tracing mode
     .. note::
@@ -1625,6 +1665,12 @@ def open_tunnel(*args, **kwargs):
     ssh_port = kwargs.pop('ssh_port', 22)
     skip_tunnel_checkup = kwargs.pop('skip_tunnel_checkup', True)
     block_on_close = kwargs.pop('block_on_close', _DAEMON)
+    if block_on_close:
+        warnings.warn("'block_on_close' is DEPRECATED. You should use either"
+                      " .stop() or .stop(force=True), depends on what you do"
+                      " with the active connections. This option has no"
+                      " affect since 0.3.0",
+                      DeprecationWarning)
     if not args:
         if isinstance(ssh_address_or_host, tuple):
             args = (ssh_address_or_host, )
@@ -1632,8 +1678,6 @@ def open_tunnel(*args, **kwargs):
             args = ((ssh_address_or_host, ssh_port), )
     forwarder = SSHTunnelForwarder(*args, **kwargs)
     forwarder.skip_tunnel_checkup = skip_tunnel_checkup
-    forwarder.daemon_forward_servers = not block_on_close
-    forwarder.daemon_transport = not block_on_close
     return forwarder
 
 
